(document:surround "/std/meta-attributes")

;;; composite widgets
(document:envelop with-container-presentations
		   ((radiolist '/std/radiolist)))

(document:envelop with-attributes (radiolist-rows radiolist-current radiolist-append-row))

;;helper for tab
(define (tab name . args)
  (tab-page name (apply vbox args)))

;;helper for vertical box
(define (vbox . args)
  (apply box orientation 'vertical args))

;;helper for horizontal box
(define (hbox . args)
  (apply box orientation 'horizontal args))

;;; loops and splashes

;;helper for splash
(define (splash-message . arg)
  (let ((arg (if (null? arg) "" (car arg))))
    (simple-notify document:root
                   'action "splash" 'value arg)))

;;helper for release
(define (document:release)
  (define-operation pause)
  (simple-notify document:root 'action "retry")
  (pause (fluid-ref generic-session)))

;;exit current dialog modal loop
(define (document:end . arg)
  (define-operation stop)
  (stop (global 'document:loop) (if (pair? arg) (car arg)))) ;;return unspecified by default

;;; message boxes

(define (document:messagebox . args)
  (define-operation pause)
  (apply simple-notify document:root 'action "messagebox" args)
  (pause (fluid-ref generic-session)))

(define (document:popup-type type title message . buttons)
  (define (default-button args)
    (if (pair? args)
        (car args)
        'no-button))

  (document:messagebox
   'type type
   'title title
   'message message
   'buttons buttons
   'default-button (default-button buttons)))

(define (document:popup-information message . buttons)
  (apply document:popup-type "information" (_ "Information" "alterator-lookout") message buttons))

(define (document:popup-warning message . buttons)
  (apply document:popup-type "warning" (_ "Warning" "alterator-lookout") message buttons))


(define (document:popup-critical message . buttons)
  (apply document:popup-type "critical" (_ "Critical" "alterator-lookout") message buttons))

(define (document:popup-question message . buttons)
  (apply document:popup-type "question" (_ "Question" "alterator-lookout") message buttons))


;;; woo-catch
(define (woo-catch/message proc)
  (woo-catch
   proc
   (lambda(reason) (document:popup-critical reason 'ok) #f)))

;;; effects: common

(define (effect-input name)
  (any (lambda(x) (or (string=? (x type) "label") x))
       (document-get-elements-by-name document:root name)))

(define (effect-input-value name)
  (let ((input (effect-input name)))
    (if input (input value) 'not-found)))

(define (effect-tlist tlist)
  (delete-duplicates (cell-ref tlist)))

(define (effect-add! slist tlist fieldname targetname targetvalue)
  (cell-set! tlist (cons targetname (cell-ref tlist)))
  (let* ((l (cell-ref slist))
	 (v (cond-assoc fieldname l '()))
	 (l (alist-delete fieldname l)))
    (cell-set! slist (alist-set fieldname (cons (cons targetname targetvalue) v) l))))

(define (effect-need? rules)
  (any (lambda(x) (equal? (effect-input-value (car x))
			  (cdr x)))
       rules))

(define (effect-check graph proc)
  (for-each (lambda(x) (proc (car x)
			     (effect-need? (cdr x))))
	    (cell-ref graph)))

(define (effect-make tlist check)
  (for-each (lambda(name)
	      (let ((w (effect-input name)))
		(and w
		     (w (when toggled (check))
			(when selected (check))))))
	    (effect-tlist tlist))
  (check))

;;; effects: disable

(define *effect-disable-slist* (make-cell '()))
(define *effect-disable-tlist* (make-cell '()))

(define (effect-disable fieldname targetname targetvalue)
  (effect-add! *effect-disable-slist*
	       *effect-disable-tlist*
	       fieldname
	       targetname
	       targetvalue))

(define (effect-disable-do name need)
  (for-each
    (lambda(w) (w activity (not need)))
    (document-get-elements-by-name document:root name)))

(define (effect-disable-check)
  (effect-check *effect-disable-slist* effect-disable-do))

;;; effects: enable

(define *effect-enable-slist* (make-cell '()))
(define *effect-enable-tlist* (make-cell '()))

(define (effect-enable fieldname targetname targetvalue)
  (effect-add! *effect-enable-slist*
	       *effect-enable-tlist*
	       fieldname
	       targetname
	       targetvalue))

(define (effect-enable-check)
  (effect-check *effect-enable-slist* (lambda (name need) (effect-disable-do name (not need)))))

;;; effects: show

(define *effect-show-slist* (make-cell '()))
(define *effect-show-tlist* (make-cell '()))

(define (effect-show fieldname targetname targetvalue)
  (effect-add! *effect-show-slist*
	       *effect-show-tlist*
	       fieldname
	       targetname
	       targetvalue))

(define (effect-show-do name need)
  (for-each
    (lambda(w) (w visibility need))
    (document-get-elements-by-name document:root name)))

(define (effect-show-check)
  (effect-check *effect-show-slist* effect-show-do))

;;; effects: hide

(define *effect-hide-slist* (make-cell '()))
(define *effect-hide-tlist* (make-cell '()))

(define (effect-hide fieldname targetname targetvalue)
  (effect-add! *effect-hide-slist*
	       *effect-hide-tlist*
	       fieldname
	       targetname
	       targetvalue))

(define (effect-hide-check)
  (effect-check *effect-hide-slist* (lambda (name need) (effect-show-do name (not need)))))

;;; effects: common

(define (init-effect)
  (effect-make *effect-disable-tlist*
               effect-disable-check)

  (effect-make *effect-enable-tlist*
               effect-enable-check)

  (effect-make *effect-show-tlist*
               effect-show-check)

  (effect-make *effect-hide-tlist*
               effect-hide-check))

(define (update-effect)
  (effect-enable-check)
  (effect-disable-check)
  (effect-show-check)
  (effect-hide-check))

;;; i18n
(define *po-domain* (make-cell (default-po-domain (or (global 'document:url) ""))))
(define *translator-cache* (make-translator-cache (session-language)))

(define (po-domain domain)
  (cell-set! *po-domain* domain))

(define (_ str . domain)
  (let  ((default-domain (cell-ref *po-domain*))
         (domain (if (pair? domain) (car domain) "_")))
    (translate *translator-cache* str domain default-domain)))
