;special wrappers over two frame buttons
(document:surround "/std/base")

(define *frame:cell-list* '(frame:apply frame:leave frame:next frame:back))

;;; cells
(define (frame:set-cell name thunk)
  (let ((cell (global name)))
       (and cell (cell-set! cell thunk))))

(define (frame:init-cell name)
  (set-global! name (make-cell #f)))

(define (frame:clear-cell name)
  (let ((cell (global name)))
    (and cell (cell-set! cell #f))))

(define (frame:call-cell name . args)
  (and-let* ((cell (global name))
             (proc (cell-ref cell)))
       (and (procedure? proc) (apply proc args))))


;;; frame page (frame:page)
(define (frame:page . args)
  (let ((page (document:subdocument)))
    (set-global! 'frame:page page)
    (apply page args)
    page))

(define (frame:replace url . args)
  (let ((page (global 'frame:page)))
     (and page
          (begin
            (for-each frame:clear-cell *frame:cell-list*)
            (frame:next-activity #t)
            (frame:back-activity #t)
            (apply document:replace-in-widget page url args)))))


;;; acc callbacks (frame:apply frame:leave)
(define (frame:on-leave thunk)
  (frame:set-cell 'frame:leave thunk))
(define (frame:clear-on-leave)
  (frame:clear-cell 'frame:leave))
(define (frame:call-on-leave)
  (frame:call-cell 'frame:leave))
(define (frame:init-on-leave)
  (frame:init-cell 'frame:leave))

(define (frame:on-apply thunk)
  (frame:set-cell 'frame:apply thunk))
(define (frame:clear-on-apply)
  (frame:clear-cell 'frame:apply))
(define (frame:call-on-apply)
  (frame:call-cell 'frame:apply))
(define (frame:init-on-apply)
  (frame:init-cell 'frame:apply))

;;; wizard callbacks (frame:next frame:back)
(define (frame:on-back thunk)
  (frame:set-cell 'frame:back thunk))
(define (frame:clear-on-back)
  (frame:clear-cell 'frame:back))
(define (frame:call-on-back)
  (frame:call-cell 'frame:back))
(define (frame:init-on-back)
  (frame:init-cell 'frame:back))

(define (frame:on-next thunk)
  (frame:set-cell 'frame:next thunk))
(define (frame:clear-on-next)
  (frame:clear-cell 'frame:next))
(define (frame:call-on-next)
  (frame:call-cell 'frame:next))
(define (frame:init-on-next)
  (frame:init-cell 'frame:next))

;;; wizard transitions (frame:next-button frame:back-button frame:end-button)
(define (frame:call-global name . args)
  (let ((p (global name)))
    (and (procedure? p) (apply p args))))

(define (frame:set-next thunk)
  (set-global! 'frame:next-button thunk))
(define (frame:next)
  (frame:call-global 'frame:next-button))

(define (frame:set-back thunk)
  (set-global! 'frame:back-button thunk))
(define (frame:back)
  (frame:call-global 'frame:back-button))

(define (frame:set-end thunk)
  (set-global! 'frame:end-button thunk))
(define (frame:end)
  (frame:call-global 'frame:end-button))

(define (frame:skip)
  (if (eq? (global 'frame:direction) 'back)
      (frame:back)
      (frame:next)))

;;; wizard buttons (frame:wizard)
(define (frame:set-wizard widget)
  (set-global! 'frame:wizard widget)
  widget)

(define (frame:set-activity name value)
  (frame:call-global 'frame:wizard action-activity name value))

(define (frame:next-activity value)
  (frame:set-activity 'forward value)
  (frame:set-activity 'finish value))

(define (frame:back-activity value)
  (frame:set-activity 'backward value))
